@@ -47,40 +47,30 @@ pub mod path {
         t.register("basename", Path::file_name);
         t.register("withext", Path::with_extension::<&str>);
         t.register("withfilename", Path::with_file_name::<&str>);
-        t.set(
-            "split",
-            cfn!(|s, path: &str| {
-                Path::new(path)
-                    .parent()
-                    .and_then(Path::to_str)
-                    .map(|dir| {
-                        let name = &path[dir.len()..];
-                        s.push(dir);
-                        s.push(name.trim_start_matches('\\'));
-                        2
-                    })
-                    .unwrap_or_default()
-            }),
-        );
-        t.set(
-            "splitext",
-            cfn!(|s, path: &str| {
-                Path::new(path)
-                    .extension()
-                    .and_then(OsStr::to_str)
-                    .map(|ext| {
-                        let p = &path[..path.len() - ext.len()];
-                        s.push(p.trim_end_matches('.'));
-                        s.push(ext);
-                        2
-                    })
-                    .unwrap_or_else(|| {
-                        s.push_value(1);
-                        s.push("");
-                        2
-                    })
-            }),
-        );
+        t.register("split", |path: &'static str| {
+            Path::new(path)
+                .parent()
+                .and_then(Path::to_str)
+                .map(|dir| {
+                    let name = &path[dir.len()..];
+                    (dir, name.trim_start_matches('\\'))
+                })
+                .unwrap_or_default()
+        });
+        t.register("splitext", |s: &State, path: &'static str| {
+            Path::new(path)
+                .extension()
+                .and_then(OsStr::to_str)
+                .map(|ext| {
+                    let p = &path[..path.len() - ext.len()];
+                    s.pushed((p.trim_end_matches('.'), ext))
+                })
+                .unwrap_or_else(|| {
+                    s.push_value(1);
+                    s.push("");
+                    Pushed(2)
+                })
+        });
         t.register("copy", std::fs::copy::<&str, &str>);
         t.register("rename", std::fs::rename::<&str, &str>);
         t.register("removedir", std::fs::remove_dir::<&str>);
@@ -89,21 +79,17 @@ pub mod path {
         // t.register("hardlink", std::fs::hard_link::<&str, &str>);
         t.register("readlink", Path::read_link);
         t.register("meta", Path::metadata);
-        t.set(
-            "join",
-            cfn!(|s, path: &Path| {
-                let mut buf = path.to_path_buf();
-                for i in 2..=s.get_top() {
-                    if let Some(n) = s.to_str(i) {
-                        buf = buf.join(n);
-                    } else {
-                        break;
-                    }
+        t.register("join", |s: &State, path: &Path| {
+            let mut buf = path.to_path_buf();
+            for i in 2..=s.get_top() {
+                if let Some(n) = s.to_str(i) {
+                    buf = buf.join(n);
+                } else {
+                    break;
                 }
-                s.push(buf.to_str());
-                1
-            }),
-        );
+            }
+            buf
+        });
         t.set("exepath", RsFn::new(std::env::current_exe));
     }
 
@@ -310,8 +296,8 @@ pub fn extend_os(s: &State) {
         use glob::glob;
         glob(pattern).map(|iter| {
             BoxIter(Box::new(
-                iter.filter_map(|e| e.ok())
-                    .filter_map(|path| path.to_str().map(|s| s.to_string())),
+                iter.filter_map(Result::ok)
+                    .map(|path| path.to_string_lossy().to_string()),
             ))
         })
     });
@@ -325,7 +311,6 @@ pub fn extend_os(s: &State) {
         };
     });
 
-    use process::*;
     use std::collections::HashMap;
     use std::process::{Command, Stdio};
 
@@ -367,57 +352,245 @@ pub fn extend_string(s: &State) {
     s.get_global(cstr!("string"));
     let string = s.val(-1);
 
-    string.set(
-        "to_utf16",
-        cfn!(|s, t: &str| {
-            let mut r = t.encode_utf16().collect::<Vec<_>>();
-            r.push(0);
-            s.push(core::slice::from_raw_parts(
-                r.as_ptr() as *const u8,
-                r.len() * 2 - 1,
-            ));
-            1
-        }),
-    );
-    string.set(
-        "from_utf16",
-        cfn!(|s, t: &[u8]| push {
-            let u = core::slice::from_raw_parts(t.as_ptr() as *const u16, t.len() / 2);
-            String::from_utf16_lossy(u)
-        }),
-    );
-    string.set(
-        "starts_with",
-        cfn!(|s, t: &str, prefix: &str| push {
-            t.starts_with(prefix)
-        }),
-    );
-    string.set(
-        "ends_with",
-        cfn!(|s, t: &str, suffix: &str| push {
-            t.ends_with(suffix)
-        }),
-    );
-    string.set(
-        "equal",
-        cfn!(|s, t1: &str, t2: &str, case_sensitive: bool| push {
-            if case_sensitive { t1.eq(t2) } else { t1.eq_ignore_ascii_case(t2) }
-        }),
-    );
-    string.set(
+    impl FromLua for glob::Pattern {
+        fn check(s: &State, i: Index) -> Self {
+            s.check_result(glob::Pattern::new(s.args(i)))
+        }
+
+        fn from_lua(s: &State, i: Index) -> Option<Self> {
+            glob::Pattern::new(s.to_str(i)?).ok()
+        }
+    }
+
+    string.register("to_utf16", |s: &State, t: &str| unsafe {
+        let mut r = t.encode_utf16().collect::<Vec<_>>();
+        r.push(0);
+        s.pushed(core::slice::from_raw_parts(
+            r.as_ptr() as *const u8,
+            r.len() * 2 - 1,
+        ))
+    });
+    string.register("from_utf16", |t: &[u8]| unsafe {
+        let u = core::slice::from_raw_parts(t.as_ptr() as *const u16, t.len() / 2);
+        String::from_utf16_lossy(u)
+    });
+    string.register("starts_with", |t: &str, prefix: &str| t.starts_with(prefix));
+    string.register("ends_with", |t: &str, suffix: &str| t.ends_with(suffix));
+    string.register("equal", |t1: &str, t2: &str, case_sensitive: bool| {
+        if case_sensitive {
+            t1.eq(t2)
+        } else {
+            t1.eq_ignore_ascii_case(t2)
+        }
+    });
+    string.register(
         "wildmatch",
-        cfn!(|s, t1: &str, pattern: &str, case_sensitive: bool|? {
-            let pattern = glob::Pattern::new(pattern)?;
-            let options = glob::MatchOptions {case_sensitive, ..Default::default()};
-            s.push(pattern.matches_with(t1, options));
-            1
-        }),
+        |t1: &str, pattern: glob::Pattern, case_sensitive: bool| {
+            let options = glob::MatchOptions {
+                case_sensitive,
+                ..Default::default()
+            };
+            pattern.matches_with(t1, options)
+        },
     );
 }
 
+#[cfg(feature = "thread")]
+mod thread {
+    use super::*;
+
+    #[cfg(not(target_os = "windows"))]
+    use std::os::unix::thread::{JoinHandleExt, RawPthread as RawHandle};
+    #[cfg(target_os = "windows")]
+    use std::os::windows::io::{AsRawHandle, RawHandle};
+    use std::ptr;
+    use std::sync::*;
+    use std::thread::{self, JoinHandle};
+    use std::time::Duration;
+
+    struct LLuaThread {
+        handle: RawHandle,
+        join: Option<JoinHandle<()>>,
+    }
+
+    impl LLuaThread {
+        #[inline]
+        fn get(&self) -> Result<&JoinHandle<()>, &'static str> {
+            self.join.as_ref().ok_or("thread joined")
+        }
+    }
+
+    impl UserData for LLuaThread {
+        const TYPE_NAME: &'static str = "LLuaThread";
+
+        fn getter(fields: &ValRef) {
+            fields.register("handle", |this: &Self| this.handle as usize);
+            fields.register("name", |this: &'static Self| {
+                this.get().map(|j| j.thread().name())
+            });
+            fields.register("id", |this: &Self| {
+                this.get().map(|j| j.thread().id().as_u64().get())
+            });
+        }
+
+        fn methods(mt: &ValRef) {
+            mt.register("join", |this: &mut Self| {
+                this.join.take().ok_or("thread joined").map(|j| j.join())
+            });
+            mt.register("unpark", |this: &Self| {
+                this.get().map(|j| j.thread().unpark())
+            });
+        }
+    }
+
+    #[cfg(target_os = "windows")]
+    const RAW_NULL: RawHandle = ptr::null_mut();
+    #[cfg(not(target_os = "windows"))]
+    const RAW_NULL: RawHandle = 0;
+
+    #[derive(Default, Deref, AsRef)]
+    struct LLuaMutex(Mutex<()>);
+    struct LLuaMutexGaurd(Option<MutexGuard<'static, ()>>);
+
+    impl UserData for LLuaMutexGaurd {
+        const TYPE_NAME: &'static str = "LLuaMutexGaurd";
+
+        fn methods(mt: &ValRef) {
+            fn unlock(this: &mut LLuaMutexGaurd) {
+                this.0.take();
+            }
+
+            mt.register("unlock", unlock);
+            mt.register("__close", unlock);
+        }
+    }
+
+    impl UserData for LLuaMutex {
+        const TYPE_NAME: &'static str = "LLuaMutex";
+
+        fn methods(mt: &ValRef) {
+            MethodRegistry::<Self, Mutex<()>>::new(mt)
+                .register("is_poisoned", Mutex::<()>::is_poisoned);
+            mt.register("lock", |this: &'static Self| {
+                this.0.lock().map(|g| LLuaMutexGaurd(Some(g)))
+            });
+            mt.register("try_lock", |this: &'static Self| {
+                this.0.try_lock().ok().map(|g| LLuaMutexGaurd(Some(g)))
+            });
+        }
+    }
+
+    #[derive(Default)]
+    struct LLuaCondVar {
+        lock: Mutex<i32>,
+        cvar: Condvar,
+    }
+
+    impl UserData for LLuaCondVar {
+        const TYPE_NAME: &'static str = "LLuaCondVar";
+
+        fn methods(mt: &ValRef) {
+            mt.register("wait", |s: &State, this: &'static Self, tm: Option<u64>| {
+                this.wait(s, tm).map(Pushed)
+            });
+            mt.register("notify_one", LLuaCondVar::notify_one);
+            mt.register("notify_all", LLuaCondVar::notify_all);
+        }
+    }
+
+    impl LLuaCondVar {
+        fn push_some(&self, s: &State) {
+            let mut i = self.lock.lock().unwrap();
+            s.unreference(LUA_REGISTRYINDEX, (*i).into());
+            s.push_value(2);
+            *i = s.reference(LUA_REGISTRYINDEX).0;
+        }
+
+        fn notify_one(&self, s: &State) {
+            self.push_some(s);
+            self.cvar.notify_one();
+        }
+
+        fn notify_all(&self, s: &State) {
+            self.push_some(s);
+            self.cvar.notify_all();
+        }
+
+        fn wait<'a>(
+            &'a self,
+            s: &State,
+            timeout: Option<u64>,
+        ) -> Result<i32, Box<dyn std::error::Error + 'a>> {
+            let lock = &self.lock;
+            let cvar = &self.cvar;
+            if let Some(tm) = timeout {
+                let (i, r) = cvar.wait_timeout(lock.lock().unwrap(), Duration::from_millis(tm))?;
+                if r.timed_out() {
+                    return Ok(0);
+                }
+                s.raw_geti(LUA_REGISTRYINDEX, (*i) as i64);
+            } else {
+                let i = cvar.wait(lock.lock().unwrap())?;
+                s.raw_geti(LUA_REGISTRYINDEX, (*i) as i64);
+            }
+            Ok(1)
+        }
+    }
+
+    pub fn init(s: &State) {
+        let t = s.table(0, 4);
+        t.register("spawn", |routine: Coroutine, name: Option<&str>| {
+            let mut b = thread::Builder::new();
+            if let Some(name) = name {
+                b = b.name(name.into());
+            }
+            b.spawn(move || {
+                if let Err(err) = routine.pcall_trace::<_, ()>(()) {
+                    call_print(
+                        &routine,
+                        &std::format!(
+                            "<thread#{} \"{}\"> {}",
+                            thread::current().id().as_u64().get(),
+                            thread::current().name().unwrap_or_default(),
+                            err
+                        ),
+                    );
+                }
+            })
+            .map(|join| {
+                #[cfg(target_os = "windows")]
+                let handle = join.as_raw_handle();
+                #[cfg(not(target_os = "windows"))]
+                let handle = join.as_pthread_t();
+                // let u = s.push_userdata(LLuaThread {join, handle, ref_ud: Cell::new(NOREF)}, Some(LLuaThread::init));
+                // u.ref_ud.set({ s.push_value(-1); s.reference(LUA_REGISTRYINDEX) });
+                // return 1;
+                // TODO: ref_ud
+                LLuaThread {
+                    join: Some(join),
+                    handle,
+                }
+            })
+        });
+        t.register("sleep", |time: u64| {
+            thread::sleep(Duration::from_millis(time))
+        });
+        t.register("park", thread::park);
+        t.register("id", || thread::current().id().as_u64().get());
+        t.register("name", |s: &State| s.pushed(thread::current().name()));
+        t.register("yield_now", thread::yield_now);
+        t.register("mutex", LLuaMutex::default);
+        t.register("condvar", LLuaCondVar::default);
+
+        s.set_global(cstr!("thread"));
+    }
+}
+
 pub fn init_global(s: &State) {
     extend_os(s);
     extend_string(s);
+    #[cfg(feature = "thread")]
+    thread::init(s);
 
     let g = s.global();
     g.set(
@@ -430,3 +603,14 @@ pub fn init_global(s: &State) {
     );
     g.register("writefile", std::fs::write::<&std::path::Path, &[u8]>);
 }
+
+pub fn call_print(s: &State, err: &str) {
+    if s.get_global(cstr!("__llua_error")) == Type::Function
+        || s.get_global(cstr!("print")) != Type::Function
+    {
+        s.push(err);
+        s.pcall(1, 0, 0);
+    } else {
+        std::eprintln!("[callback error] {}", err);
+    }
+}
