@@ -1,13 +1,12 @@
 use super::*;
+use crate::{ffi::*, lua_Integer as Integer, lua_Number as Number, str::*};
 
-use crate::{ffi::*, lua_Integer as Integer, lua_Number as Number};
+use alloc::boxed::Box;
 use alloc::sync::Arc;
 use core::fmt::Debug;
 use core::marker::PhantomData;
 use core::mem;
-use core::ops::Deref;
 use libc::c_int;
-use std::process::Output;
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
 pub struct CRegRef(pub i32);
@@ -40,18 +39,39 @@ impl From<i32> for Pushed {
     }
 }
 
+fn get_weak_meta(s: &State) {
+    let top = s.get_top();
+    s.push_light_userdata(get_weak_meta as usize as *mut ());
+    if s.c_reg().get(TopVal).type_of() != Type::Table {
+        s.pop(1);
+        s.new_table();
+        s.val(-1).set("__mode", "v");
+        s.push_light_userdata(get_weak_meta as usize as *mut ());
+        s.push_value(-2);
+        s.raw_set(LUA_REGISTRYINDEX);
+    }
+    assert_eq!(s.get_top(), top + 1);
+}
+
 pub trait UserData: Sized {
     /// `__name`
     const TYPE_NAME: &'static str = core::any::type_name::<Self>();
-    /// set `__index` to metatable itself
-    const INDEX_SELF: bool = true;
-    /// `__len` metamethod, if true, return the size of this userdata
+
+    /// get value from metatable itself in `__index`
+    const INDEX_METATABLE: bool = true;
+
+    /// get value from the first uservalue in `__index`
+    const INDEX_USERVALUE: bool = false;
+
+    const INDEX_GETTER: lua_CFunction = None;
+
+    /// set the `__len` metamethod, if true, return the size of this userdata
     const RAW_LEN: bool = false;
 
-    const INDEX_USERVALUE: bool = false;
-    const GETTER: lua_CFunction = None;
     const IS_POINTER: bool = false;
 
+    const WEAK_REF_CACHE: bool = true;
+
     /// add methods
     fn methods(mt: &ValRef) {}
 
@@ -85,6 +105,69 @@ pub trait UserData: Sized {
         Self::methods(&mt);
     }
 
+    #[inline(always)]
+    fn metatable() -> InitMetatable {
+        Self::init_metatable
+    }
+
+    /// initialize userdata on the top of lua stack
+    fn init_userdata(s: &State) {
+        if Self::INDEX_USERVALUE {
+            s.new_table();
+            s.set_uservalue(-2);
+        }
+    }
+
+    /// get a pointer whose type is lightuserdata as the key in cache table
+    fn key_to_cache(&self) -> *const () {
+        core::ptr::null()
+    }
+
+    fn clear_cached(&self, s: &State) {
+        s.get_or_init_metatable(Self::init_metatable);
+        assert!(s.get_metatable(-1));
+        let key = self.key_to_cache();
+        s.push_light_userdata(key as usize as *mut ());
+        s.push_nil();
+        s.raw_set(-3);
+        s.pop(2);
+    }
+
+    fn get_cahced(s: &State, key: *const ()) -> bool {
+        s.get_or_init_metatable(Self::init_metatable);
+        // use metatable of userdata's metatable as cache table
+        if !s.get_metatable(-1) {
+            s.new_table();
+            s.push_value(-1);
+            s.set_metatable(-3);
+            if Self::WEAK_REF_CACHE {
+                get_weak_meta(s);
+                s.set_metatable(-2);
+            }
+        }
+        s.push_light_userdata(key as usize as *mut ());
+        if s.raw_get(-2) == Type::Userdata {
+            s.replace(-3);
+            s.pop(1);
+            return true;
+        }
+        s.pop(1);
+        s.push_light_userdata(key as usize as *mut ());
+        false
+    }
+
+    fn cache_userdata(s: &State, _key: *const ()) {
+        // meta | meta's meta | key | userdata
+        s.push_value(-1);
+        s.replace(-5);
+        s.raw_set(-3);
+        s.pop(1);
+    }
+
+    fn uservalue_count(&self, s: &State) -> i32 {
+        Self::INDEX_USERVALUE as _
+    }
+
     unsafe extern "C" fn __index(l: *mut lua_State) -> c_int {
         let s = State::from_ptr(l);
 
@@ -98,23 +181,25 @@ pub trait UserData: Sized {
         }
 
         // access method table
-        if Self::INDEX_SELF && !s.get_metatable_by(1, s.val(2)).is_none_or_nil() {
+        if Self::INDEX_METATABLE && !s.get_metatable_by(1, s.val(2)).is_none_or_nil() {
             return 1;
         }
 
-        // access getter function
-        if let Some(getter) = Self::GETTER {
-            let n = getter(l);
-            if n > 0 {
-                return n;
-            }
-        }
-
         // access user value as table
         if Self::INDEX_USERVALUE {
             s.get_uservalue(1);
             s.push_value(2);
-            s.get_table(-2);
+            if !s.get_table(-2).is_none_or_nil() {
+                return 1;
+            }
+        }
+
+        // access getter function
+        if let Some(getter) = Self::INDEX_GETTER {
+            s.push(getter);
+            s.push_value(1);
+            s.push_value(2);
+            s.call(2, 1);
             return 1;
         }
 
@@ -171,19 +256,28 @@ pub trait UserData: Sized {
         0
     }
 }
-// impl<T> UserData for Arc<T> {}
 
 impl<T: UserData> ToLua for T {
     #[inline(always)]
     fn to_lua(self, s: &State) {
+        let key = self.key_to_cache();
+        if !key.is_null() && T::get_cahced(s, key) {
+            return;
+        }
+
         if T::IS_POINTER {
             s.push_userdata_pointer_body(self, Self::init_metatable);
         } else {
-            s.push_userdata(self, Some(Self::init_metatable));
+            let count = self.uservalue_count(s);
+            s.push_userdatauv(self, count);
+            s.set_or_init_metatable(Self::init_metatable);
         }
         if T::INDEX_USERVALUE {
-            s.new_table();
-            s.set_uservalue(-2);
+            s.balance_with(T::init_userdata);
+        }
+
+        if !key.is_null() {
+            T::cache_userdata(s, key)
         }
     }
 }
@@ -192,27 +286,37 @@ impl<T: UserData> ToLua for *mut T {
     #[inline(always)]
     fn to_lua(self, s: &State) {
         assert!(T::IS_POINTER);
-        s.push_userdata_pointer(self, T::init_metatable);
-        if T::INDEX_USERVALUE {
-            s.new_table();
-            s.set_uservalue(-2);
+
+        if let Some(r) = unsafe { self.as_ref() } {
+            let key = r.key_to_cache();
+            if !key.is_null() && T::get_cahced(s, key) {
+                return;
+            }
+            s.push_userdata_pointer(self, T::init_metatable);
+            if T::INDEX_USERVALUE {
+                s.balance_with(T::init_userdata);
+            }
+            if !key.is_null() {
+                T::cache_userdata(s, key)
+            }
+        } else {
+            s.push_nil();
         }
     }
 }
 
-pub trait LuaFn<'a, THIS, ARGS, RET> {
+pub trait LuaFn<'a, THIS: 'a, ARGS: 'a, RET: 'a> {
     unsafe extern "C" fn wrapper(l: *mut lua_State) -> c_int;
 }
 
-impl<'a, THIS: 'a, T, O, F: LuaFn<'a, THIS, T, O>> RsFn<THIS, T, O, F> {
+impl<'a, THIS: 'a, T: 'a, O: 'a, F: LuaFn<'a, THIS, T, O>> RsFn<THIS, T, O, F> {
     pub const fn wrapper(&self) -> lua_CFunction {
-        // TODO:
-        // assert_eq!(core::mem::size_of::<F>(), 0);
+        assert!(core::mem::size_of::<F>() == 0);
         Some(F::wrapper)
     }
 }
 
-impl<'a, T, O, F: LuaFn<'a, (), T, O>> RsFn<(), T, O, F> {
+impl<'a, T: 'a, O: 'a, F: LuaFn<'a, (), T, O>> RsFn<(), T, O, F> {
     pub const fn new(f: F) -> Self {
         Self(f, PhantomData)
     }
@@ -251,13 +355,21 @@ impl<'a> ToLua for &'a std::ffi::OsStr {
     }
 }
 
-impl ToLua for String {
-    #[inline(always)]
-    fn to_lua(self, s: &State) {
-        s.push_string(&self);
-    }
+macro_rules! impl_as_str {
+    ($t:ty) => {
+        impl ToLua for $t {
+            #[inline(always)]
+            fn to_lua(self, s: &State) {
+                s.push_string(self.as_ref());
+            }
+        }
+    };
 }
 
+impl_as_str!(Arc<str>);
+impl_as_str!(Box<str>);
+impl_as_str!(String);
+
 impl<'a> ToLua for &'a [u8] {
     #[inline(always)]
     fn to_lua(self, s: &State) {
@@ -305,7 +417,7 @@ impl<T> ToLua for UserDataWrapper<T> {
 impl<T: ToLua, I: Iterator<Item = T>> ToLua for IterVec<T, I> {
     #[inline(always)]
     fn to_lua(self, s: &State) {
-        let r = s.table(self.0.size_hint().0 as _, 0);
+        let r = s.table(self.0.size_hint().1.unwrap_or(0) as _, 0);
         let mut i = 1;
         for e in self.0.into_iter() {
             r.seti(i, e);
@@ -317,7 +429,7 @@ impl<T: ToLua, I: Iterator<Item = T>> ToLua for IterVec<T, I> {
 impl<K: ToLua, V: ToLua, I: Iterator<Item = (K, V)>> ToLua for IterMap<K, V, I> {
     #[inline(always)]
     fn to_lua(self, s: &State) {
-        let r = s.table(0, self.0.size_hint().0 as i32);
+        let r = s.table(0, self.0.size_hint().1.unwrap_or(0) as _);
         for (k, v) in self.0 {
             r.set(k, v);
         }
@@ -325,6 +437,10 @@ impl<K: ToLua, V: ToLua, I: Iterator<Item = (K, V)>> ToLua for IterMap<K, V, I>
 }
 
 impl<'a, T: ToLuaMulti> BoxIter<'a, T> {
+    pub fn new(iter: impl Iterator<Item = T> + 'a) -> Self {
+        Self(Box::new(iter))
+    }
+
     unsafe extern "C" fn lua_fn(l: *mut lua_State) -> c_int {
         let s = State::from_ptr(l);
         let p = s.to_userdata(ffi::lua_upvalueindex(1));
@@ -337,16 +453,25 @@ impl<'a, T: ToLuaMulti> BoxIter<'a, T> {
     }
 }
 
+unsafe extern "C" fn __gc<T>(l: *mut lua_State) -> i32 {
+    let s = State::from_ptr(l);
+    s.to_userdata_typed::<T>(1)
+        .map(|p| core::ptr::drop_in_place(p));
+    return 0;
+}
+
 impl<'a, T: ToLuaMulti> ToLua for BoxIter<'a, T> {
     #[inline(always)]
     fn to_lua(self, s: &State) {
-        type BoxIterT = BoxIter<'static, usize>;
-        s.push_userdata(self, Some(metatable!([s: State, this: BoxIterT])));
-        s.push_cclosure(Some(BoxIter::<'a, T>::lua_fn), 1);
+        s.push_userdata(self, None);
+        let mt = s.table(0, 1);
+        mt.set("__gc", __gc::<BoxIter<'static, usize>> as CFunction);
+        s.set_metatable(-2);
+        s.push_cclosure(Some(Self::lua_fn), 1);
     }
 }
 
-impl<'a, THIS, T, O, F: LuaFn<'a, THIS, T, O>> ToLua for RsFn<THIS, T, O, F> {
+impl<'a, THIS: 'a, T: 'a, O: 'a, F: LuaFn<'a, THIS, T, O>> ToLua for RsFn<THIS, T, O, F> {
     #[inline(always)]
     fn to_lua(self, s: &State) {
         if core::mem::size_of::<Self>() == 0 {
@@ -356,8 +481,10 @@ impl<'a, THIS, T, O, F: LuaFn<'a, THIS, T, O>> ToLua for RsFn<THIS, T, O, F> {
             let pfptr = &self;
             s.push_light_userdata(unsafe { *mem::transmute::<_, *const *mut ()>(pfptr) });
         } else {
-            // TODO: metatable __gc
             s.push_userdatauv(self, 0);
+            let mt = s.table(0, 1);
+            mt.set("__gc", __gc::<Self> as CFunction);
+            s.set_metatable(-2);
         };
         s.push_cclosure(Some(F::wrapper), 1);
     }
@@ -787,7 +914,7 @@ macro_rules! impl_luafn {
             }
         }
 
-        impl<'a, FN: Fn(&State, $($x,)*)->RET + 'static, $($x: FromLua,)* RET: ToLuaMulti> LuaFn<'a, (), (State, $($x,)*), RET> for FN {
+        impl<'a, FN: Fn(&State, $($x,)*)->RET + 'static, $($x: FromLua+'a,)* RET: ToLuaMulti+'a> LuaFn<'a, (), (State, $($x,)*), RET> for FN {
             unsafe extern "C" fn wrapper(l: *mut lua_State) -> c_int {
                 let s = State::from_ptr(l);
                 let f: &Self;
@@ -796,7 +923,7 @@ macro_rules! impl_luafn {
             }
         }
 
-        impl<'a, FN: Fn(THIS, &State, $($x,)*)->RET + 'static, THIS: FromLua, $($x: FromLua,)* RET: ToLuaMulti> LuaFn<'a, (), (THIS, State, $($x,)*), RET> for FN {
+        impl<'a, FN: Fn(THIS, &State, $($x,)*)->RET + 'static, THIS: FromLua+'a, $($x: FromLua+'a,)* RET: ToLuaMulti+'a> LuaFn<'a, (), (THIS, State, $($x,)*), RET> for FN {
             unsafe extern "C" fn wrapper(l: *mut lua_State) -> c_int {
                 let s = State::from_ptr(l);
                 let f: &Self;
@@ -806,7 +933,7 @@ macro_rules! impl_luafn {
         }
 
         #[allow(unused_parens)]
-        impl<'a, FN: for<'r> Fn(&'r T $(,$x)*)->RET, T: ?Sized, THIS: UserData+AsRef<T>+'a, $($x: FromLua,)* RET: ToLuaMulti> LuaFn<'a, (THIS, &'a T), ($($x,)*), RET> for FN {
+        impl<'a, FN: for<'r> Fn(&'r T $(,$x)*)->RET, T: ?Sized, THIS: UserData+AsRef<T>+'a, $($x: FromLua+'a,)* RET: ToLuaMulti+'a> LuaFn<'a, (THIS, &'a T), ($($x,)*), RET> for FN {
             unsafe extern "C" fn wrapper(l: *mut lua_State) -> c_int {
                 let s = State::from_ptr(l);
                 let f: &Self;
@@ -817,7 +944,7 @@ macro_rules! impl_luafn {
         }
 
         #[allow(unused_parens)]
-        impl<'a, FN: for<'r> Fn(&'r mut T $(,$x)*)->RET, T: ?Sized, THIS: UserData+AsMut<T>+'a, $($x: FromLua,)* RET: ToLuaMulti> LuaFn<'a, (THIS, &'a mut T), ($($x,)*), RET> for FN {
+        impl<'a, FN: for<'r> Fn(&'r mut T $(,$x)*)->RET, T: ?Sized, THIS: UserData+AsMut<T>+'a, $($x: FromLua+'a,)* RET: ToLuaMulti+'a> LuaFn<'a, (THIS, &'a mut T), ($($x,)*), RET> for FN {
             unsafe extern "C" fn wrapper(l: *mut lua_State) -> c_int {
                 let s = State::from_ptr(l);
                 let f: &Self;
@@ -849,11 +976,18 @@ impl State {
     pub fn pushed<T: ToLuaMulti>(&self, t: T) -> Pushed {
         Pushed(self.pushx(t))
     }
+
+    /// [-0, +1, -]
+    #[inline(always)]
+    pub fn metatable<U: UserData>(&self) -> ValRef {
+        self.get_or_init_metatable(U::init_metatable);
+        self.val(-1)
+    }
 }
 
 impl ValRef<'_> {
     #[inline(always)]
-    pub fn register<'a, K: ToLua, V: LuaFn<'a, (), ARGS, RET>, ARGS, RET>(
+    pub fn register<'a, K: ToLua, V: LuaFn<'a, (), ARGS, RET>, ARGS: 'a, RET: 'a>(
         &self,
         k: K,
         v: V,
@@ -874,7 +1008,7 @@ where
     }
 
     #[inline]
-    pub fn register<K, V, ARGS, RET>(&self, k: K, v: V) -> &Self
+    pub fn register<K, V, ARGS: 'b, RET: 'b>(&self, k: K, v: V) -> &Self
     where
         K: ToLua,
         V: LuaFn<'b, (T, &'b D), ARGS, RET>,
@@ -897,7 +1031,7 @@ where
     }
 
     #[inline]
-    pub fn register<K, V, ARGS, RET>(&self, k: K, v: V) -> &Self
+    pub fn register<K, V, ARGS: 'b, RET: 'b>(&self, k: K, v: V) -> &Self
     where
         K: ToLua,
         V: LuaFn<'b, (T, &'b mut D), ARGS, RET>,
