@@ -1,15 +1,13 @@
 use super::*;
 
 use crate::convert::{FromLua, FromLuaMulti, ToLua, ToLuaMulti};
-use crate::ffi::*;
+use crate::{ffi::*, str::*};
 
 use alloc::format;
 use alloc::string::String;
 use alloc::vec::Vec;
 use core::mem::MaybeUninit;
-use core::ops::DerefMut;
-use core::{any, mem, ptr, slice, str};
-
+use core::{mem, ptr, slice, str};
 use libc::{c_char, c_int, c_void, size_t};
 
 pub type InitMetatable = fn(&ValRef);
@@ -220,10 +218,6 @@ bitflags::bitflags! {
 #[repr(C)]
 pub struct State(*mut lua_State);
 
-extern "C" {
-    fn llua_open_libs(_: &State);
-}
-
 impl State {
     /// Initializes a new Lua state. This function does not open any libraries
     /// by default. Calls `lua_newstate` internally.
@@ -253,7 +247,6 @@ impl State {
     pub fn open_libs(&self) {
         unsafe {
             luaL_openlibs(self.0);
-            llua_open_libs(self);
         }
     }
 
@@ -721,6 +714,13 @@ impl State {
         Type::from_c_int(result)
     }
 
+    /// [-0, +1, –] Maps to `lua_getiuservalue`.
+    #[inline(always)]
+    pub fn get_iuservalue(&self, idx: Index, n: i32) -> Type {
+        let result = unsafe { lua_getiuservalue(self.0, idx, n) };
+        Type::from_c_int(result)
+    }
+
     //===========================================================================
     // Set functions (stack -> Lua)
     //===========================================================================
@@ -786,6 +786,14 @@ impl State {
         }
     }
 
+    /// [-1, +0, -] Maps to `lua_setiuservalue`.
+    #[inline(always)]
+    pub fn set_iuservalue(&self, idx: Index, n: i32) {
+        unsafe {
+            lua_setiuservalue(self.0, idx, n);
+        }
+    }
+
     //===========================================================================
     // 'load' and 'call' functions (load and run Lua code)
     //===========================================================================
@@ -1322,7 +1330,6 @@ impl State {
 
     /// Maps to `luaL_checkoption`.
     pub fn check_option(&self, arg: Index, def: Option<&str>, lst: &[&str]) -> usize {
-        use libc::c_char;
         let mut vec: Vec<*const c_char> = Vec::with_capacity(lst.len() + 1);
         let cstrs: Vec<CString> = lst.iter().map(|ent| CString::new(*ent).unwrap()).collect();
         for ent in cstrs.iter() {
@@ -1467,8 +1474,8 @@ impl State {
 
     /// Maps to `luaL_requiref`.
     #[inline(always)]
-    pub fn requiref(&self, modname: &CStr, openf: lua_CFunction, glb: bool) {
-        unsafe { luaL_requiref(self.0, modname.as_ptr(), openf, glb as c_int) }
+    pub fn requiref(&self, modname: &CStr, openf: CFunction, glb: bool) {
+        unsafe { luaL_requiref(self.0, modname.as_ptr(), Some(openf), glb as c_int) }
     }
 
     /// Maps to `luaL_newlibtable`.
@@ -1593,17 +1600,28 @@ impl State {
     }
 
     /// [-0, +1, -]
-    pub fn push_userdata_pointer<T>(&self, data: *mut T, metatable: InitMetatable) {
-        let result: &mut *mut T =
-            unsafe { mem::transmute(self.new_userdata(mem::size_of_val(&data))) };
+    pub fn push_userdata_pointer<T: UserData>(&self, data: *mut T, metatable: InitMetatable) {
+        let result: &mut *mut T = unsafe {
+            mem::transmute(self.new_userdatauv(
+                mem::size_of_val(&data),
+                data.as_ref().unwrap().uservalue_count(self),
+            ))
+        };
         mem::replace(result, data);
         self.set_or_init_metatable(metatable);
     }
 
     /// [-0, +1, -]
-    pub fn push_userdata_pointer_body<T>(&self, data: T, metatable: InitMetatable) -> &mut T {
-        let result: &mut (*mut T, T) =
-            unsafe { mem::transmute(self.new_userdata(mem::size_of::<(*mut T, T)>())) };
+    pub fn push_userdata_pointer_body<T: UserData>(
+        &self,
+        data: T,
+        metatable: InitMetatable,
+    ) -> &mut T {
+        let result: &mut (*mut T, T) = unsafe {
+            mem::transmute(
+                self.new_userdatauv(mem::size_of::<(*mut T, T)>(), data.uservalue_count(self)),
+            )
+        };
         mem::forget(mem::replace(result, (ptr::null_mut(), data)));
         result.0 = &mut result.1;
         self.set_or_init_metatable(metatable);
